#ifndef CGIARGS_H
#define CGIARGS_H


#ifdef __GNUG__
#  include <ostream.h>
#else
#  ifndef USE_OBJECTSPACE
#    include <ostream>
#  else
#    include <ospace\std\ostream>
#  endif
#endif

#ifndef USE_OBJECTSPACE
#  include <map>
#else
#  include <ospace\std\map>
#endif


#include "text_t.h"

typedef map<text_t, text_t, lttext_t> textmap;


class cgiargsclass
{
protected:
  textmap args;

public:
  //type support for textmap
  typedef textmap::iterator iterator;
  typedef textmap::const_iterator const_iterator;
  typedef textmap::reference reference;
  typedef textmap::const_reference const_reference;
  typedef textmap::size_type size_type;
  typedef textmap::difference_type difference_type;
  typedef textmap::const_reverse_iterator const_reverse_iterator;
  typedef textmap::reverse_iterator reverse_iterator;
  
  // constructors
  cgiargsclass ();

  // basic container support
  iterator begin () {return args.begin();}
  const_iterator begin () const {return args.begin();}
  iterator end () {return args.end();}
  const_iterator end () const {return args.end();}

  void erase(iterator pos) {args.erase(pos);}
  void erase(iterator first, iterator last) {args.erase(first, last);}
  cgiargsclass &operator=(const cgiargsclass &x) {args=x.args;return *this;}

  bool empty () const {return args.empty();}
  size_type size() const {return args.size();}


  // added functionality
  void clear () {args.erase(args.begin(),args.end());}

  // setdefaultarg, setdefaultintarg, and setdefaultcarg will 
  // only set the argument if it is not already set
  // getargs NULLs if there isn't an entry with
  // 'key' already defined, getintarg returns 0 if there wasn't an
  // entry with 'key' defined and operator[] returns "" if
  // 'key' wasn't already defined (and sets 'key'="").
  void setarg (const text_t &key, const text_t &value);
  void setdefaultarg (const text_t &key, const text_t &value); 
  void setintarg (const text_t &key, int value);
  void setdefaultintarg (const text_t &key, int value);
  void setcarg (const text_t &key, unsigned short c);
  void setdefaultcarg (const text_t &key, unsigned short c);
  text_t *getarg (const text_t &key);
  int getintarg (const text_t &key);
  text_t &operator[] (const text_t &key) {return args[key];}
};


// stream operators to print cgi arguments for debugging purposes
ostream &operator<<(ostream &outs, const cgiargsclass &args);


#endif
